module junyou{
            
    function hash(str:String):number
    {
        var len:number=str.length;
        var hval:number=0, g:number;
        var hash:number=5381;
        for (var i=0; i < len; i++)
        {
            hash+=(hash << 5) + str.charCodeAt(i);
        }
        return hash & 0x7fffffff;
    }
    
    interface Path{
        path:string,
        iPrefix?:boolean
    }
    
    interface JConfig { 
        params?: {},
        prefixes: string[],
        paths: { 
            res: Path,
            skin:Path
        };
    }
    
    export class ConfigUtils{

        /**
         * 配置数据
         */
        private static _data: JConfig;
        
        /**
         * 资源的hash配置
         */
        private static _hash:{[index: string]: string};
        
        
        public static setData(data: JConfig){
            this._data=data;
        }
        
        /**
         * 获取资源完整路径
         * @param key 资源标识
         */
        public static getResUrl(uri:string,local:Boolean=false):string{
            if(local){
                return "resource/" + uri;
            }
            var hash = this._hash;
            if(hash){
                var ver:string=hash[uri];
                if (ver)
				{
					if (uri.indexOf("?") == -1)
					{
						uri=uri + "?" + ver;
					}
					else
					{
						uri=uri + "&jyver=" + ver;
					}
				}
            }
            return this.getUrlWithPath(uri,this._data.paths.res);
        }
        
        private static getUrlWithPath(uri:string,path:Path){
            uri = path.path + uri;
            var prefix = path.iPrefix ? "" : this.getPrefix(uri);
            return prefix + uri;
        }
        
        /**
         * 根据资源标识获取网址前缀
         */
        public static getPrefix(uri:string):string{
            var prefixes = this._data.prefixes;
            var idx=hash(uri)%prefixes.length;
            return prefixes[idx]||"";
        }
        
        /**
         * 获取参数
         */
        public static getParam(key:string):any{
            return this._data.params[key];
        }
        
        /**
         * 获取皮肤文件地址
         */ 
        public static getSkinFile(key:string,fileName:string){
            return this.getUrlWithPath(key + "/" + fileName,this._data.paths.skin);
        }

    }
    
    export function createObjectFromArray<T>(ref: { new (): T},keys:string[],data:any[]):T {
        var obj = new ref();
        for(let i = 0,len = keys.length;i < len;i++){
            let key = keys[i];
            if(key in obj){
                if(i in data) {
                    obj[key] = data[i];
                }
            }
        }
        return obj;
    }
}