module junyou.game{
     /**
     * MapRender
     * 用于处理地图平铺的渲染
     */
    export class TileMapLayer extends GameLayer {
        
        /**
         * @private
         */
        currentMap:MapInfo;
        
        
        public setRect(rect:egret.Rectangle){
            var cM = this.currentMap;
            if(!cM){
                return;
            }
            //检查地图，进行加载区块
            var x = rect.x;
            var y = rect.y;
            var w = rect.width;
            var h = rect.height;
            
            var pW = cM.pWidth;
            var pH = cM.pHeight;
            var sc=x/pW>>0;
            var sr=y/pH>>0;
            var ec = (x + w)/pW>>0;
            var er = (y+h)/pH>>0;
            ec = Math.min(ec,cM.maxPicX);
            er = Math.min(er,cM.maxPicY);
            for(let r=sr;r<=er;r++){
                for(let c=sc;c<=ec;c++){
                    let uri = cM.getMapUri(c,r);
                    let tm = <TileMap>ResourceManager.getResource(uri);
                    if(!tm){
                        tm = new TileMap();
                        tm.reset(c,r,uri);
                        tm.x = c * pW;
                        tm.y = r * pH;
                        if(!ResourceManager.regResource(uri,tm)){
                            throw Error("资源注册失败");
                        }
                        tm.load();
                        this.addChild(tm);                
                    }
                    tm.lastUseTime=Date.now();                    
                }
            }
        }
        
        constructor(id:number) {
            super(id)
        }
    }
    
     /**
     * TileMap
     */
    export class TileMap extends BaseResource {
        /**
         * 地图块的列
         */
        private col:number;
        /**
         * 地图块的行
         */
        private row:number;

        /**
         * 资源唯一标识
         */
        private uri:string;

        
        public lastUseTime:number;
        
        get resID(){
            return this.uri;
        }
        
        
        constructor() {
            super();       
        }        
        
        reset(col: number,row: number,uri: string) {
            this.col = col;
            this.row = row;
            this.uri = uri;
            this.url = ConfigUtils.getResUrl(uri);
        }

    }
}