module junyou.game {
    
    enum ValueMessageType{
        Null=0,
        Boolean=1,
        Number=2,
        String=3
    }
    
    import ByteArray = egret.ByteArray;
	/**
	 * 通信服务
	 * 收发的协议结构：
	 * 2字节协议号 2字节包长度(n) n字节包
	 * @author 
	 *
	 */
	export class NetService extends egret.EventDispatcher{
    	
    	protected _router:NetRouter;
    	protected _limiter:RequestLimit;
    	
    	/**
    	 * 被动指令池
    	 */ 
    	protected _sendDataPool:RecyclablePool<NetSendData>;
    	
    	/**
    	 * 待发送的的被动指令列表
    	 */ 
    	protected _pcmdList:NetSendData[];
    	
    	
    /**
     * 接收数据的临时数组
     */ 
    protected _tmpList: NetData[];
    /**
     * 接收数据的处理回收池
     */ 
    protected _netDataPool: RecyclablePool<NetData>;
    	
    	/**
    	 * 读取的字节缓存
    	 */ 
      protected _readBuffer:ByteArray;
      
      /**
       * 发送的字节缓存
       */ 
      protected _sendBuffer:ByteArray;
      

      protected _tempBytes: ByteArray;
      
      /**
       * 接收消息的创建器
       * 
       */ 
      protected _recieveMSG: { [index: number]: { new (): IMessage } | ValueMessageType};
      
      /**
       * 是否对接收的数据打印日志
       */ 
      protected _isLogRecieve:boolean;
      
      /**
       * 是否对发送的数据打印日志
       */ 
      protected _isLogSend:boolean;
      
      /**
       * 认证信息
       */ 
      protected _authData: AuthData;
    	
    	constructor(){
        super();
    	  this._router = new NetRouter();
        this._limiter = new RequestLimit();
        this._sendDataPool = new RecyclablePool(NetSendData);
        this._pcmdList = [];
        this._tmpList=[];
        this._readBuffer = new ByteArray();
        this._sendBuffer = new ByteArray();
        this._tempBytes = new ByteArray();
        this._recieveMSG = {};
    	}
    	
    	/**
    	 * 设置认证信息
    	 */ 
    	public setAuthData(data:AuthData){
            this._authData = data;
    	}
    	
    	public login(){
    	    
    	}
    	
    	/**
    	 * 基础类型消息
    	 */ 
        public regRecieveMSGRef(cmd: number,ref: { new (): IMessage } | ValueMessageType){
            this._recieveMSG[cmd]=ref;
        }
    	
    	/**
    	 * 注册处理器
    	 * @param {number} cmd 协议号
    	 * @param {INetHandler} handler 处理网络数据的处理器
    	 * @param {number} priority 处理优先级
    	 */ 
       public register(cmd:number,handler:INetHandler,priotity:number=0):boolean{
           return this._registerListener(cmd,handler,priotity,false);
    	 }
    	
    	/**
    	 * 注册单次执行的处理器
    	 * @param {number} cmd 协议号
    	 * @param {INetHandler} handler 处理网络数据的处理器
    	 * @param {number} priority 处理优先级
    	 */ 
       public regOnce(cmd: number,handler: INetHandler,priotity: number = 0):boolean{
           return this._registerListener(cmd,handler,priotity,true);
       }
       
       protected _registerListener(cmd: number,handler: INetHandler,priotity:number,once:boolean):boolean{
           if(cmd > 32767 || cmd < -32768) {
               ThrowError("协议号的范围必须是-32768~32767之间",0,1);
               return false;
           }
           this._router.registerListener(cmd,handler,priotity,once);
           return true;
       }
    	
    	/**
    	 * 即时发送指令<br/>
    	 * 用于处理角色主动操作的请求，如点击合成道具，使用物品等
    	 * @param {number} cmd 协议号
    	 * @param {any} data 发送的数据
    	 * @param {number} limit 客户端发送时间限制
    	 */ 
       public send(cmd: number,data?: (number | boolean | string | IMessage),limit?:number){
            
            if(this._limiter.check(cmd,limit)){
                this._send(cmd,data);
            }
        }
    	
        /**
         * 即时发送指令
         */ 
        protected _send(cmd: number,data: (number | boolean | string | IMessage)){
            //to be override
        }
        
        /**
         * 断开连接
         */ 
        public disconnect(){
            //TODO
        }
        
        /**
         * 消极发送指令<br/>
         * 如果使用通协议的指令，有堆积的指令，先合并，新的替换旧的<br/>
         * <font color='#FF0000'>请勿将一些用户操作使用此指令发送</font>
         * 处理一些实时性要求不高的指令，这些指令先缓存堆积，等到用户主动发指令的时候，一起发送<br/>
         * @param {number} cmd 协议号
         * @param {any} data 发送的数据
         */ 
        public sendPassive(cmd: number,data?: (number | boolean | string | IMessage)){
            //合并同协议的指令
            var pcmdList = this._pcmdList;
            var len = pcmdList.length;
            for(let temp of pcmdList){
                if(temp.cmd == cmd){
                    temp.data = data;
                    return;
                }
            }
            //没有同协议的指令，新增数据
            var pdata = this._sendDataPool.getInstance();
            pdata.cmd=cmd;
            pdata.data=data;
            //将数据缓存在pcmdList中，等到下次send的时候发送
            this._pcmdList[len]=pdata;
        }

        /**
         * 向缓冲数组中写入数据
         */ 
        protected writeToBuffer(bytes:ByteArray,data:NetSendData){
            var cmd = data.cmd;
            var dat = data.data;
            bytes.writeShort(cmd);
            if(!dat){
                return;
            }
            var type = typeof dat;
            switch(type) {
                case "boolean":
                    bytes.writeBoolean(<boolean>dat);
                    break;
                case "number":
                    bytes.writeDouble(<number>dat);
                    break;
                case "string":
                    bytes.writeUTF(<string>dat);
                    break;
                default:
                    var tempBytes = this._tempBytes;
                    tempBytes.clear();
                    var msg = <IMessage>dat;
                    msg.writeTo(tempBytes);
                    bytes.writeShort(tempBytes.length);
                    bytes.writeBytes(bytes);
                    break;
            }
        }
        
        /**
         * @private 
         * @param bytes
         * @param out
         * @param 是否发生错误 true 解析时发生错误
         *                    false 正常解析
         */
        protected decodeBytes(bytes:ByteArray):boolean {
            
            var recieveMSG = this._recieveMSG;
            var recievePool = this._netDataPool;
            var tmpList = this._tmpList;
            var idx=0;
            var error=false;
            while(bytes.bytesAvailable) {
                //先读取2字节协议号
                let cmd = bytes.readShort();
                let type = recieveMSG[cmd];
                if(type){
                    let data;
                    switch(type){
                        case ValueMessageType.Null:
                            data=null;
                            break;
                        case ValueMessageType.Boolean:
                            data=bytes.readBoolean();
                            break;
                        case ValueMessageType.Number:
                            data=bytes.readDouble();
                            break;
                        case ValueMessageType.String:
                            data=bytes.readUTF();
                            break;
                        default:
                            //先读取2字节消息长度
                            let len = bytes.readShort();
                            var tempBytes = this._tempBytes;
                            tempBytes.clear();
                            bytes.readBytes(tempBytes,len);
                            var ref = <{new():IMessage}>type;
                            var msg = new ref();
                            msg.readFrom(tempBytes,len);
                            data=msg;
                            break;
                    }
                    let nData = recievePool.getInstance();
                    nData.cmd=cmd;
                    nData.data = data;
                    tmpList[idx++]=nData;
                }else{
                    error = true;
                    break;
                }
            }
            
            //调试时,显示接收的数据
            if(DEBUG){
                var now = Date.now();
                if(this._isLogRecieve){
                    for(let nData of tmpList){                        
                        Logger.debug("recieve:\t"+now+"\t"+nData.cmd+"\t"+ JSON.stringify(nData.data));
                    }
                }
            }
            
            var router = this._router;
            //分发数据
            for(let nData of tmpList) {
                router.dispatch(nData);
                //回收nData
                recievePool.recycle(nData);
            }
            tmpList.length=0;
            return error;
        }
	}

}
