module junyou.game {

    const LITTLE_ENDIAN = egret.Endian.LITTLE_ENDIAN;
    const TYPE_DOUBLE: number = 1;
    const TYPE_FLOAT: number = 2;
    const TYPE_INT64: number = 3;
    const TYPE_UINT64: number = 4;
    const TYPE_INT32: number = 5;
    const TYPE_FIXED64: number = 6;
    const TYPE_FIXED32: number = 7;
    const TYPE_BOOL: number = 8;
    const TYPE_STRING: number = 9;
    const TYPE_GROUP: number = 10;
    const TYPE_MESSAGE: number = 11;
    const TYPE_BYTES: number = 12;
    const TYPE_UINT32: number = 13;
    const TYPE_ENUM: number = 14;
    const TYPE_SFIXED32: number = 15;
    const TYPE_SFIXED64: number = 16;
    const TYPE_SINT32: number = 17;
    const TYPE_SINT64: number = 18;
	/**
	 *
	 * @author 3tion
	 * javascript 只会使用到 varint32->number string boolean
	 *
	 */
    export class PBMessageUtils {

        public static readFrom(msg: PBMessage,bytes: ByteArray,len: number = 0): void {
            var affterLen: number = 0;
            if(len > 0)
                affterLen = bytes.bytesAvailable - len;
            while(bytes.bytesAvailable > affterLen) {
                var tag: number = bytes.readVarint();
                if(tag == 0)
                    continue;
                var number: number = tag >>> 3;
                var body: Object = msg.mMessageEncode[number];
                if(body) {
                    var name: string = body[0];
                    var label: number = body[1];
                    var type: number = body[2];
                    var typeObj: PBMessageConstructor = body[3];
                }
                var value: Object;
                switch(type) {
                    case TYPE_DOUBLE:
                        value = bytes.readDouble();
                        break;
                    case TYPE_FLOAT:
                        value = bytes.readFloat();
                        break;
                    case TYPE_INT64:
                    case TYPE_UINT64:
                    case TYPE_SINT64:
                        value = bytes.readVarint64();
                        break;
                    case TYPE_INT32:
                    case TYPE_UINT32:
                    case TYPE_ENUM:
                    case TYPE_SINT32:
                        value = bytes.readVarint();
                        break;
                    case TYPE_FIXED64:
                    case TYPE_SFIXED64:
                        value = this.readFix64(bytes);//理论上项目不使用
                        break;
                    case TYPE_FIXED32:
                        value = bytes.readUnsignedInt();
                        break;
                    case TYPE_BOOL:
                        value = this.readBoolean(bytes);
                        break;
                    case TYPE_STRING:
                        value = this.readString(bytes);
                        break;
                    case TYPE_GROUP://(已弃用)
                        value = null;
                        break;
                    case TYPE_MESSAGE://消息
                        value = this.readMessage(tag,bytes,typeObj);
                        break;
                    case TYPE_BYTES:
                        value = this.readBytes(tag,bytes);
                        break;
                    case TYPE_SFIXED32:
                        value = bytes.readInt();
                        break;
                    default:
                        value = this.readValue(tag,bytes);
                }
                if(body) {
                    if(label == 3) {
                        msg[name].push(value);
                    }
                    else {
                        msg[name] = value;
                    }
                }
            }
        }

        private static readValue(tag: number,bytes: ByteArray): any {
            var wrieType: number = tag & 7;
            var value: any;
            switch(wrieType) {
                case 0: //Varint	int32, int64, uint32, uint64, sint32, sint64, bool, enum
                    value = bytes.readVarint();
                    break;
                case 2: //Length-delimi	string, bytes, embedded messages, packed repeated fields
                    value = this.readString(bytes);
                    break;
                case 5: //32-bit	fixed32, sfixed32, float
                    value = bytes.readInt();
                    break;
                case 1: //64-bit	fixed64, sfixed64, double
                    value = bytes.readDouble();
                    break;
                //case 3://Start group	Groups (deprecated)
                //break;
                //case 4://End group	Groups (deprecated)
                //break;
                default:
                    ThrowError("错误");
            }
            return value;
        }


        private static readFix64(bytes: ByteArray): Object {
            var v:Int64 = new Int64;
            v.low = bytes.readUnsignedInt();
            v.high = bytes.readUnsignedInt();
            return v;
        }


        private static readBoolean(bytes: ByteArray): Object {
            return bytes.readVarint() > 0;
        }

        private static readString(bytes: ByteArray): Object {
            var blen: number = bytes.readVarint();
            if(blen > 0) {
                var temp: ByteArray = new ByteArray;
                temp.endian = LITTLE_ENDIAN;
                bytes.readBytes(temp,0,blen);
                temp.position = 0;
                return temp.readUTFBytes(temp.length);
            }
            return "";
        }


        private static readMessage(tag: number,bytes: ByteArray,typeObj: PBMessageConstructor): Object {
            var blen: number = bytes.readVarint();
            var msg: PBMessage = new typeObj;
            this.readFrom(msg,bytes,blen);
            return msg;
        }

        private static readBytes(tag: number,bytes: ByteArray): Object {
            var blen: number = bytes.readVarint();
            var temp: ByteArray = new ByteArray;
            temp.endian = LITTLE_ENDIAN;
            if(blen != 0) {
                bytes.readBytes(temp,0,blen);
            }
            return temp;
        }




        public static writeTo(msg: PBMessage,bytes: ByteArray=null): ByteArray {
            var messageEncode: Object = msg.mMessageEncode;
            if(messageEncode == null) {
                return null;
            }
            bytes = bytes || new ByteArray;
            bytes.endian = LITTLE_ENDIAN;
            for(let numberStr in messageEncode) {
                var number = +numberStr;
                var body: Object = messageEncode[number];
                var label: number = body[1];
                if(label == 1 && !msg.has(number)) {
                    continue;
                }
                var name: string = body[0];
                var value: Object = msg[name];
                if(value == null) {
                    continue;
                }
                var type: number = body[2];

                var wrieType: number = this.type2WrieType(type);
                var tag: number = (number << 3) | wrieType;
                if(label == 3) {
                    for (let key in value)
                    {
                        var element=value[key];
                        this.writeElementTo(element,type,tag,bytes);
                    }
                }
                else {
                    this.writeElementTo(value,type,tag,bytes);
                }
            }
            return bytes;
        }

        public static writeElementTo(value:any,type: number,tag: number,bytes: ByteArray): void {
            bytes.writeVarint(tag);
            switch(type) {
                case TYPE_FIXED32:
                    bytes.writeInt(value as number);
                    break;
                case TYPE_SFIXED32:
                    bytes.writeUnsignedInt(value as number);
                    break;
                case TYPE_FLOAT:
                    bytes.writeFloat(value as number);
                    break;
                case TYPE_DOUBLE:
                case TYPE_FIXED64:
                case TYPE_SFIXED64:
                    bytes.writeDouble(value as number);
                    break;
                case TYPE_INT32:
                case TYPE_SINT32:
                case TYPE_ENUM:
                case TYPE_UINT32:
                    bytes.writeVarint(value as number);
                    break;
                case TYPE_INT64:
                case TYPE_SINT64:
                case TYPE_UINT64:
                    bytes.writeVarint64(value as Int64);
                    break;
                case TYPE_BOOL:
                    bytes.writeVarint(value ? 1 : 0);
                    break;
                case TYPE_STRING:
                case TYPE_BYTES:
                case TYPE_MESSAGE:
                    if(type == TYPE_MESSAGE) {
                        var temp: ByteArray = this.writeTo(<PBMessage>value);
                    }
                    else if(type == TYPE_BYTES) {
                        temp = value as ByteArray;
                    }
                    else {
                        temp = new ByteArray;
                        temp.endian = LITTLE_ENDIAN;
                        temp.writeUTFBytes(value as string);
                    }
                    bytes.writeVarint(temp.length);
                    bytes.writeBytes(temp,0,temp.length);
            }
        }


        public static type2WrieType(type: number): number {
            switch(type) {
                case TYPE_FIXED32:
                case TYPE_SFIXED32:
                case TYPE_FLOAT:
                    return 5;
                case TYPE_DOUBLE:
                case TYPE_FIXED64:
                case TYPE_SFIXED64:
                    return 1;
                case TYPE_INT32:
                case TYPE_SINT32:
                case TYPE_ENUM:
                case TYPE_UINT32:
                case TYPE_INT64:
                case TYPE_SINT64:
                case TYPE_UINT64:
                case TYPE_BOOL:
                    return 0;
                case TYPE_STRING:
                case TYPE_MESSAGE:
                case TYPE_BYTES:
                    return 2;
            }
            return -1;
        }

    }
}

